//  Copyright (c) 2018-2019
//  Cem Bassoy
//
//  Distributed under the Boost Software License, Version 1.0. (See
//  accompanying file LICENSE_1_0.txt or copy at
//  http://www.boost.org/LICENSE_1_0.txt)
//
//  The authors gratefully acknowledge the support of
//  Fraunhofer and Google in producing this work
//  which started as a Google Summer of Code project.
//

/// \file tensor.hpp Definition for the class vector and its derivative

#ifndef BOOST_NUMERIC_UBLAS_TENSOR_HPP
#define BOOST_NUMERIC_UBLAS_TENSOR_HPP

#include "tensor/functions.hpp"
#include "tensor/operators_arithmetic.hpp"
#include "tensor/operators_comparison.hpp"
#include "tensor/extents.hpp"
#include "tensor/strides.hpp"
#include "tensor/ostream.hpp"
#include "tensor/tensor.hpp"

#endif // BOOST_NUMERIC_UBLAS_TENSOR_HPP
